@@ -16,7 +16,7 @@
 #ifndef RADIUSD_H
 #define RADIUSD_H
 /**
- * $Id: 37e50183b44c819adb3fa8cd59518f708af05419 $
+ * $Id: 9cfa418f228aeeb60f2c52b433211c38ceadfaea $
  *
  * @file radiusd.h
  * @brief Structures, prototypes and global variables for the FreeRADIUS server.
@@ -24,7 +24,7 @@
  * @copyright 1999-2000,2002-2008  The FreeRADIUS server project
  */
 
-RCSIDH(radiusd_h, "$Id: 37e50183b44c819adb3fa8cd59518f708af05419 $")
+RCSIDH(radiusd_h, "$Id: 9cfa418f228aeeb60f2c52b433211c38ceadfaea $")
 
 #include <freeradius-devel/libradius.h>
 #include <freeradius-devel/radpaths.h>
@@ -141,6 +141,8 @@ typedef struct main_config {
 	uint32_t	cleanup_delay;			//!< How long before cleaning up cached responses.
 	uint32_t	max_requests;
 
+	uint32_t       	proxy_dedup_window;		//!< suppress duplicate retransmitssions from a NAS
+
 	bool		postauth_client_lost;		//!< Whether to run Post-Auth-Type Client-Lost section
 
 	uint32_t	debug_level;
@@ -174,6 +176,9 @@ typedef struct main_config {
 
 	bool		exiting;			//!< are we exiting?
 
+	bool		require_ma;			//!< global configuration for all clients and home servers
+
+	bool		limit_proxy_state;     		//!< global configuration for all clients
 
 #ifdef ENABLE_OPENSSL_VERSION_CHECK
 	char const	*allow_vulnerable_openssl;	//!< The CVE number of the last security issue acknowledged.
@@ -277,6 +282,8 @@ struct rad_request {
 
 	RAD_LISTEN_TYPE		priority;
 
+	bool			max_time;	//!< did we hit max time?
+
 	bool			in_request_hash;
 #ifdef WITH_PROXY
 	bool			in_proxy_hash;
@@ -355,8 +362,8 @@ extern char const	*radacct_dir;
 extern char const	*radlog_dir;
 extern char const	*radlib_dir;
 extern bool		log_stripped_names;
-extern char const	*radiusd_version;
-extern char const	*radiusd_version_short;
+extern HIDDEN char const	*radiusd_version;
+extern HIDDEN char const	*radiusd_version_short;
 void			radius_signal_self(int flag);
 
 typedef enum {
@@ -378,9 +385,9 @@ int		rad_accounting(REQUEST *);
 int		rad_coa_recv(REQUEST *request);
 
 /* session.c */
-int		rad_check_ts(fr_ipaddr_t const *nas_addr, uint32_t nas_port, char const *user, char const *sessionid);
+int		rad_check_ts(fr_ipaddr_t const *nas_addr, uint32_t nas_port, char const *nas_port_id, char const *user, char const *sessionid);
 int		session_zap(REQUEST *request, fr_ipaddr_t const *nas_addr,
-			    uint32_t nas_port, char const *user,
+			    uint32_t nas_port, char const *nas_port_id, char const *user,
 			    char const *sessionid, uint32_t cliaddr,
 			    char proto, int session_time);
 
@@ -543,6 +550,7 @@ int	total_active_threads(void);
 void	thread_pool_lock(void);
 void	thread_pool_unlock(void);
 void	thread_pool_queue_stats(int array[RAD_LISTEN_MAX], int pps[2]);
+void	thread_pool_thread_stats(int stats[3]);
 
 #ifndef HAVE_PTHREAD_H
 #  define rad_fork(n) fork()
@@ -607,6 +615,13 @@ int dual_tls_recv(rad_listen_t *listener);
 int dual_tls_send(rad_listen_t *listener, REQUEST *request);
 int proxy_tls_recv(rad_listen_t *listener);
 int proxy_tls_send(rad_listen_t *listener, REQUEST *request);
+#ifdef WITH_COA_TUNNEL
+int proxy_tls_send_reply(rad_listen_t *listener, REQUEST *request);
+int dual_tls_send_coa_request(rad_listen_t *listener, REQUEST *request);
+void listen_coa_free(void);
+void listen_coa_add(rad_listen_t *listener, char const *key);
+int listen_coa_find(REQUEST *request, char const *key);
+#endif
 #endif
 
 /*
