@@ -5,11 +5,11 @@
  * realms.h	Structures, prototypes and global variables
  *		for realms
  *
- * Version:	$Id: 354f2a3840bf7031058587f17e4099b8d16d5003 $
+ * Version:	$Id: 93830b522957097ad1cf79c0cedae5770f409f17 $
  *
  */
 
-RCSIDH(realms_h, "$Id: 354f2a3840bf7031058587f17e4099b8d16d5003 $")
+RCSIDH(realms_h, "$Id: 93830b522957097ad1cf79c0cedae5770f409f17 $")
 
 #ifdef __cplusplus
 extern "C" {
@@ -25,6 +25,11 @@ typedef enum {
 
 #ifdef WITH_COA
 	HOME_TYPE_COA,		//!< CoA destination (NAS or Proxy)
+
+#ifdef WITH_COA_TUNNEL
+	HOME_TYPE_AUTH_COA,	//!< auth + coa
+	HOME_TYPE_AUTH_ACCT_COA, //!< auth + acct + coa
+#endif
 #endif
 } home_type_t;
 
@@ -36,14 +41,16 @@ typedef enum {
 } home_ping_check_t;
 
 typedef enum {
-	HOME_STATE_UNKNOWN = 0,
-	HOME_STATE_ALIVE,
+	HOME_STATE_ALIVE = 0,
 	HOME_STATE_ZOMBIE,
 	HOME_STATE_IS_DEAD,
-	HOME_STATE_CONNECTION_FAIL,
+	HOME_STATE_UNKNOWN,
 	HOME_STATE_ADMIN_DOWN,
+	HOME_STATE_CONNECTION_FAIL,
 } home_state_t;
 
+#define HOME_SERVER_IS_DEAD(_x) (((_x)->state == HOME_STATE_IS_DEAD) || ((_x)->state == HOME_STATE_ADMIN_DOWN) || ((_x)->state == HOME_STATE_CONNECTION_FAIL))
+
 typedef struct fr_socket_limit_t {
 	uint32_t	max_connections;
 	uint32_t	num_connections;
@@ -51,6 +58,8 @@ typedef struct fr_socket_limit_t {
 	uint32_t	num_requests;
 	uint32_t	lifetime;
 	uint32_t	idle_timeout;
+	uint32_t	read_timeout;
+	uint32_t	write_timeout;
 } fr_socket_limit_t;
 
 typedef struct home_server {
@@ -61,6 +70,12 @@ typedef struct home_server {
 
 	bool			dual;			//!< One of a pair of homeservers on consecutive ports.
 	bool			dynamic;		//!< is this a dynamically added home server?
+	bool			nonblock;		//!< Enable a socket non-blocking to the home server.
+	bool			require_ma;		//!< for all replies to Access-Request and Status-Server
+
+#ifdef WITH_COA_TUNNEL
+	bool			recv_coa;		//!< receive CoA packets, too
+#endif
 	char const		*virtual_server;		//!< For internal proxying
 	char const		*parent_server;
 
@@ -121,9 +136,14 @@ typedef struct home_server {
 	uint32_t		coa_mrc;
 	uint32_t		coa_mrt;
 	uint32_t		coa_mrd;
+#ifdef WITH_COA_TUNNEL
+	char const		*recv_coa_server;      	//!< for accepting incoming CoA requests
+#endif
 #endif
 #ifdef WITH_TLS
 	fr_tls_server_conf_t	*tls;
+	uint32_t		connect_timeout;
+	rbtree_t		*listeners;
 #endif
 
 #ifdef WITH_STATS
