@@ -5,7 +5,7 @@
  *		write a decent parser. I know how to do that, really :)
  *		miquels@cistron.nl
  *
- * Version:	$Id: bddb923af54ade4b4bf18b8e8f6f64af6382391c $
+ * Version:	$Id: ad5a5fef85aa195cc8fe08ddb786e0563e23ff7c $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -26,7 +26,7 @@
  * Copyright 2000  Alan DeKok <aland@ox.org>
  */
 
-RCSID("$Id: bddb923af54ade4b4bf18b8e8f6f64af6382391c $")
+RCSID("$Id: ad5a5fef85aa195cc8fe08ddb786e0563e23ff7c $")
 
 #include <freeradius-devel/radiusd.h>
 #include <freeradius-devel/parser.h>
@@ -127,7 +127,7 @@ typedef struct cf_file_t {
 } cf_file_t;
 
 CONF_SECTION *root_config = NULL;
-bool cf_new_escape = false;
+bool cf_new_escape = true;
 
 
 static int		cf_data_add_internal(CONF_SECTION *cs, char const *name, void *data,
@@ -353,6 +353,7 @@ error:
 
 	file->filename = filename;
 	file->cs = cs;
+	file->from_dir = from_dir;
 
 	if (fstat(fd, &file->buf) == 0) {
 #ifdef S_IWOTH
@@ -1423,12 +1424,13 @@ int cf_item_parse(CONF_SECTION *cs, char const *name, unsigned int type, void *d
 {
 	int rcode;
 	bool deprecated, required, attribute, secret, file_input, cant_be_empty, tmpl, multi, file_exists;
+	bool ignore_dflt;
 	char **q;
 	char const *value;
 	CONF_PAIR *cp = NULL;
 	fr_ipaddr_t *ipaddr;
-	char buffer[8192];
 	CONF_ITEM *c_item;
+	char buffer[8192];
 
 	if (!cs) {
 		cf_log_err(&(cs->item), "No enclosing section for configuration item \"%s\"", name);
@@ -1446,6 +1448,7 @@ int cf_item_parse(CONF_SECTION *cs, char const *name, unsigned int type, void *d
 	cant_be_empty = (type & PW_TYPE_NOT_EMPTY);
 	tmpl = (type & PW_TYPE_TMPL);
 	multi = (type & PW_TYPE_MULTI);
+	ignore_dflt = (type & PW_TYPE_IGNORE_DEFAULT);
 
 	if (attribute) required = true;
 	if (required) cant_be_empty = true;	/* May want to review this in the future... */
@@ -1469,7 +1472,7 @@ int cf_item_parse(CONF_SECTION *cs, char const *name, unsigned int type, void *d
 	 *	section, use the default value.
 	 */
 	if (!cp) {
-		if (deprecated) return 0;	/* Don't set the default value */
+		if (deprecated || ignore_dflt) return 0;	/* Don't set the default value */
 
 		rcode = 1;
 		value = dflt;
@@ -1657,6 +1660,62 @@ int cf_item_parse(CONF_SECTION *cs, char const *name, unsigned int type, void *d
 				cf_log_err(&(cs->item),"Failed expanding variable %s", name);
 				return -1;
 			}
+
+		} else if (cf_new_escape && (cp->rhs_type == T_DOUBLE_QUOTED_STRING) && (strchr(value, '\\') != NULL)) {
+			char const *p = value;
+			char *s = buffer;
+			char *end = buffer + sizeof(buffer);
+			unsigned int x;
+
+			/*
+			 *	We pass !cf_new_escape() to gettoken() when we parse the RHS of a CONF_PAIR
+			 *	above.  But gettoken() unescapes the \", and doesn't unescape anything else.
+			 *	So we do it here.
+			 */
+			while (*p && (s < end)) {
+				if (*p != '\\') {
+					*(s++) = *(p++);
+					continue;
+				}
+
+				p++;
+
+				switch (*p) {
+				case 'r':
+					*s++ = '\r';
+					break;
+				case 'n':
+					*s++ = '\n';
+					break;
+				case 't':
+					*s++ = '\t';
+					break;
+
+				default:
+					if (*p >= '0' && *p <= '9' &&
+					    sscanf(p, "%3o", &x) == 1) {
+						if (!x) {
+							cf_log_err(&(cs->item), "Cannot have embedded zeros in value for %s", name);
+							return -1;
+						}
+
+						*s++ = x;
+						p += 2;
+					} else
+						*s++ = *p;
+					break;
+				}
+				p++;
+			}
+
+			if (s == end) {
+				cf_log_err(&(cs->item), "Failed expanding value for %s", name);
+				return -1;
+			}
+
+			*s = '\0';
+
+			value = buffer;
 		}
 
 		if (cant_be_empty && (value[0] == '\0')) goto cant_be_empty;
@@ -1803,6 +1862,7 @@ static void cf_section_parse_init(CONF_SECTION *cs, void *base,
 				  CONF_PARSER const *variables)
 {
 	int i;
+	void *data;
 
 	for (i = 0; variables[i].name != NULL; i++) {
 		if (variables[i].type == PW_TYPE_SUBSECTION) {
@@ -1827,9 +1887,13 @@ static void cf_section_parse_init(CONF_SECTION *cs, void *base,
 				subcs->item.lineno = cs->item.lineno;
 				cf_item_add(cs, &(subcs->item));
 			}
+			if (base) {
+				data = ((uint8_t *)base) + variables[i].offset;
+			} else {
+				data = NULL;
+			}
 
-			cf_section_parse_init(subcs, (uint8_t *)base + variables[i].offset,
-					      (CONF_PARSER const *) variables[i].dflt);
+			cf_section_parse_init(subcs, data, (CONF_PARSER const *) variables[i].dflt);
 			continue;
 		}
 
@@ -1927,8 +1991,13 @@ int cf_section_parse(CONF_SECTION *cs, void *base, CONF_PARSER const *variables)
 				goto finish;
 			}
 
-			ret = cf_section_parse(subcs, (uint8_t *)base + variables[i].offset,
-					       (CONF_PARSER const *) variables[i].dflt);
+			if (base) {
+				data = ((uint8_t *)base) + variables[i].offset;
+			} else {
+				data = NULL;
+			}
+
+			ret = cf_section_parse(subcs, data, (CONF_PARSER const *) variables[i].dflt);
 			if (ret < 0) goto finish;
 			continue;
 		} /* else it's a CONF_PAIR */
@@ -2318,7 +2387,7 @@ static int cf_section_read(char const *filename, int *lineno, FILE *fp,
 
 		if (has_spaces) {
 			ptr = cbuf;
-			while (isspace((int) *ptr)) ptr++;
+			while (isspace((uint8_t) *ptr)) ptr++;
 
 			if (ptr > cbuf) {
 				memmove(cbuf, ptr, len - (ptr - cbuf));
@@ -2334,7 +2403,7 @@ static int cf_section_read(char const *filename, int *lineno, FILE *fp,
 			if (at_eof) break;
 
 			ptr = buf;
-			while (*ptr && isspace((int) *ptr)) ptr++;
+			while (*ptr && isspace((uint8_t) *ptr)) ptr++;
 
 			if (!*ptr || (*ptr == '#')) continue;
 
@@ -2524,8 +2593,8 @@ static int cf_section_read(char const *filename, int *lineno, FILE *fp,
 					 *	Check for valid characters
 					 */
 					for (p = dp->d_name; *p != '\0'; p++) {
-						if (isalpha((int)*p) ||
-						    isdigit((int)*p) ||
+						if (isalpha((uint8_t)*p) ||
+						    isdigit((uint8_t)*p) ||
 						    (*p == '-') ||
 						    (*p == '_') ||
 						    (*p == '.')) continue;
@@ -2805,7 +2874,7 @@ static int cf_section_read(char const *filename, int *lineno, FILE *fp,
 		case T_OP_EQ:
 		case T_OP_SET:
 		case T_OP_PREPEND:
-			while (isspace((int) *ptr)) ptr++;
+			while (isspace((uint8_t) *ptr)) ptr++;
 
 			/*
 			 *	Be a little more forgiving.
@@ -2827,7 +2896,7 @@ static int cf_section_read(char const *filename, int *lineno, FILE *fp,
 
 				t3 = T_BARE_WORD;
 				while (*q && (*q >= ' ') && (*q != ',') &&
-				       !isspace(*q)) q++;
+				       !isspace((uint8_t) *q)) q++;
 
 				if ((size_t) (q - ptr) >= sizeof(buf3)) {
 					ERROR("%s[%d]: Parse error: value too long",
@@ -2895,21 +2964,10 @@ static int cf_section_read(char const *filename, int *lineno, FILE *fp,
 			cpn->pass2 = pass2;
 			cf_item_add(this, &(cpn->item));
 
-			/*
-			 *	Hacks for escaping
-			 */
-			if (!cf_new_escape && !this->item.parent && value &&
-			    (strcmp(buf1, "correct_escapes") == 0) &&
-			    ((strcmp(value, "true") == 0) ||
-			     (strcmp(value, "yes") == 0) ||
-			     (strcmp(value, "1") == 0))) {
-				cf_new_escape = true;
-			}
-
 			/*
 			 *	Require a comma, unless there's a comment.
 			 */
-			while (isspace(*ptr)) ptr++;
+			while (isspace((uint8_t) *ptr)) ptr++;
 
 			if (*ptr == ',') {
 				ptr++;
@@ -2982,7 +3040,7 @@ static int cf_section_read(char const *filename, int *lineno, FILE *fp,
 		/*
 		 *	Done parsing one thing.  Skip to EOL if possible.
 		 */
-		while (isspace(*ptr)) ptr++;
+		while (isspace((uint8_t) *ptr)) ptr++;
 
 		if (*ptr == '#') continue;
 
