@@ -1,7 +1,7 @@
 /*
  * stats.c	Internal statistics handling.
  *
- * Version:	$Id: a5c672e365582c102f85db7dabff363670195f49 $
+ * Version:	$Id: 2f5298d384847a3eab9c928581b7fccb95e2dcbc $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -21,7 +21,7 @@
  * Copyright 2008  Alan DeKok <aland@deployingradius.com>
  */
 
-RCSID("$Id: a5c672e365582c102f85db7dabff363670195f49 $")
+RCSID("$Id: 2f5298d384847a3eab9c928581b7fccb95e2dcbc $")
 
 #include <freeradius-devel/radiusd.h>
 #include <freeradius-devel/rad_assert.h>
@@ -525,8 +525,8 @@ void request_stats_reply(REQUEST *request)
 	/*
 	 *	Authentication.
 	 */
-	if (((flag->vp_integer & 0x01) != 0) &&
-	    ((flag->vp_integer & 0xc0) == 0)) {
+	if (((flag->vp_integer & 0x01) != 0) &&		/* auth */
+	    ((flag->vp_integer & 0xe0) == 0)) {		/* not client, server or home-server */
 		request_stats_addvp(request, authvp, &radius_auth_stats);
 	}
 
@@ -534,8 +534,8 @@ void request_stats_reply(REQUEST *request)
 	/*
 	 *	Accounting
 	 */
-	if (((flag->vp_integer & 0x02) != 0) &&
-	    ((flag->vp_integer & 0xc0) == 0)) {
+	if (((flag->vp_integer & 0x02) != 0) &&		/* accounting */
+	    ((flag->vp_integer & 0xe0) == 0)) {		/* not client, server or home-server */
 		request_stats_addvp(request, acctvp, &radius_acct_stats);
 	}
 #endif
@@ -544,8 +544,8 @@ void request_stats_reply(REQUEST *request)
 	/*
 	 *	Proxied authentication requests.
 	 */
-	if (((flag->vp_integer & 0x04) != 0) &&
-	    ((flag->vp_integer & 0x20) == 0)) {
+	if (((flag->vp_integer & 0x04) != 0) &&		/* proxy-auth */
+	    ((flag->vp_integer & 0x20) == 0)) {		/* not client */
 		request_stats_addvp(request, proxy_authvp, &proxy_auth_stats);
 	}
 
@@ -553,8 +553,8 @@ void request_stats_reply(REQUEST *request)
 	/*
 	 *	Proxied accounting requests.
 	 */
-	if (((flag->vp_integer & 0x08) != 0) &&
-	    ((flag->vp_integer & 0x20) == 0)) {
+	if (((flag->vp_integer & 0x08) != 0) &&		/* proxy-accounting */
+	    ((flag->vp_integer & 0x20) == 0)) {		/* not client */
 		request_stats_addvp(request, proxy_acctvp, &proxy_acct_stats);
 	}
 #endif
@@ -563,7 +563,7 @@ void request_stats_reply(REQUEST *request)
 	/*
 	 *	Internal server statistics
 	 */
-	if ((flag->vp_integer & 0x10) != 0) {
+	if ((flag->vp_integer & 0x10) != 0) {		/* internal */
 		vp = radius_pair_create(request->reply, &request->reply->vps,
 				       PW_FREERADIUS_STATS_START_TIME, VENDORPEC_FREERADIUS);
 		if (vp) vp->vp_date = start_time.tv_sec;
@@ -607,7 +607,7 @@ void request_stats_reply(REQUEST *request)
 	/*
 	 *	For a particular client.
 	 */
-	if ((flag->vp_integer & 0x20) != 0) {
+	if ((flag->vp_integer & 0x20) != 0) { 		/* client */
 		fr_ipaddr_t ipaddr;
 		VALUE_PAIR *server_ip, *server_port = NULL;
 		RADCLIENT *client = NULL;
@@ -764,8 +764,8 @@ void request_stats_reply(REQUEST *request)
 	/*
 	 *	For a particular "listen" socket.
 	 */
-	if (((flag->vp_integer & 0x40) != 0) &&
-	    ((flag->vp_integer & 0x03) != 0)) {
+	if (((flag->vp_integer & 0x40) != 0) &&		/* server */
+	    ((flag->vp_integer & 0x03) != 0)) {		/* auth or accounting */
 		rad_listen_t *this;
 		VALUE_PAIR *server_ip, *server_port;
 		fr_ipaddr_t ipaddr;
@@ -807,7 +807,7 @@ void request_stats_reply(REQUEST *request)
 		fr_pair_add(&request->reply->vps,
 			fr_pair_copy(request->reply, server_port));
 
-		if ((flag->vp_integer & 0x01) != 0) {
+		if ((flag->vp_integer & 0x01) != 0) {	/* auth */
 			if ((request->listener->type == RAD_LISTEN_AUTH) ||
 			    (request->listener->type == RAD_LISTEN_NONE)) {
 				request_stats_addvp(request, authvp, &this->stats);
@@ -817,7 +817,7 @@ void request_stats_reply(REQUEST *request)
 		}
 
 #ifdef WITH_ACCOUNTING
-		if ((flag->vp_integer & 0x02) != 0) {
+		if ((flag->vp_integer & 0x02) != 0) {	/* accounting */
 			if ((request->listener->type == RAD_LISTEN_ACCT) ||
 			    (request->listener->type == RAD_LISTEN_NONE)) {
 				request_stats_addvp(request, acctvp, &this->stats);
@@ -832,8 +832,8 @@ void request_stats_reply(REQUEST *request)
 	/*
 	 *	Home servers.
 	 */
-	if (((flag->vp_integer & 0x80) != 0) &&
-	    ((flag->vp_integer & 0x03) != 0)) {
+	if (((flag->vp_integer & 0x80) != 0) &&		/* home-server */
+	    ((flag->vp_integer & 0x03) != 0)) {		/* auth or accounting */
 		home_server_t *home;
 		VALUE_PAIR *server_ip, *server_port;
 		fr_ipaddr_t ipaddr;
@@ -935,7 +935,7 @@ void request_stats_reply(REQUEST *request)
 				       PW_FREERADIUS_STATS_LAST_PACKET_SENT, VENDORPEC_FREERADIUS);
 		if (vp) vp->vp_date = home->last_packet_sent;
 
-		if ((flag->vp_integer & 0x01) != 0) {
+		if ((flag->vp_integer & 0x01) != 0) {	/* auth */
 			if (home->type == HOME_TYPE_AUTH) {
 				request_stats_addvp(request, proxy_authvp,
 						    &home->stats);
@@ -945,7 +945,7 @@ void request_stats_reply(REQUEST *request)
 		}
 
 #ifdef WITH_ACCOUNTING
-		if ((flag->vp_integer & 0x02) != 0) {
+		if ((flag->vp_integer & 0x02) != 0) {	/* accounting */
 			if (home->type == HOME_TYPE_ACCT) {
 				request_stats_addvp(request, proxy_acctvp,
 						    &home->stats);
@@ -991,14 +991,14 @@ void radius_stats_ema(fr_stats_ema_t *ema,
 	}
 
 
-	tdiff = start->tv_sec;
-	tdiff -= end->tv_sec;
+	tdiff = end->tv_sec;
+	tdiff -= start->tv_sec;
 
 	micro = (int) tdiff;
 	if (micro > 40) micro = 40; /* don't overflow 32-bit ints */
 	micro *= USEC;
-	micro += start->tv_usec;
-	micro -= end->tv_usec;
+	micro += end->tv_usec;
+	micro -= start->tv_usec;
 
 	micro *= EMA_SCALE;
 
