@@ -1,7 +1,7 @@
 /*
  * mem.c  Memory allocation, deallocation stuff.
  *
- * Version:     $Id: 5bfb8e61888a30cbd57e8e239c4eb4a4e6a4e7fd $
+ * Version:     $Id: 546eff3463c05ae4615baa2221c600d96f9aad09 $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -21,12 +21,15 @@
  * Copyright 2001  hereUare Communications, Inc. <raghud@hereuare.com>
  */
 
-#include <freeradius-devel/ident.h>
-RCSID("$Id: 5bfb8e61888a30cbd57e8e239c4eb4a4e6a4e7fd $")
+RCSID("$Id: 546eff3463c05ae4615baa2221c600d96f9aad09 $")
 
 #include <stdio.h>
 #include "rlm_eap.h"
 
+#ifdef WITH_TLS
+#include <freeradius-devel/tls.h>
+#endif
+
 #ifdef HAVE_PTHREAD_H
 #define PTHREAD_MUTEX_LOCK pthread_mutex_lock
 #define PTHREAD_MUTEX_UNLOCK pthread_mutex_unlock
@@ -36,64 +39,20 @@ RCSID("$Id: 5bfb8e61888a30cbd57e8e239c4eb4a4e6a4e7fd $")
 #endif
 
 /*
- * Allocate a new EAP_PACKET
+ * Allocate a new eap_packet_t
  */
-EAP_PACKET *eap_packet_alloc(void)
-{
-	EAP_PACKET   *rp;
-
-	rp = rad_malloc(sizeof(EAP_PACKET));
-	memset(rp, 0, sizeof(EAP_PACKET));
-	return rp;
-}
-
-/*
- * Free EAP_PACKET
- */
-void eap_packet_free(EAP_PACKET **eap_packet_ptr)
-{
-	EAP_PACKET *eap_packet;
-
-	if (!eap_packet_ptr) return;
-	eap_packet = *eap_packet_ptr;
-	if (!eap_packet) return;
-
-   	if (eap_packet->type.data) {
-		/*
-		 *	There's no packet, OR the type data isn't
-		 *	pointing inside of the packet: free it.
-		 */
-		if ((eap_packet->packet == NULL) ||
-		    (eap_packet->type.data != (eap_packet->packet + 5))) {
-			free(eap_packet->type.data);
-		}
-		eap_packet->type.data = NULL;
-	}
-
-	if (eap_packet->packet) {
-		free(eap_packet->packet);
-		eap_packet->packet = NULL;
-	}
-
-	free(eap_packet);
-
-	*eap_packet_ptr = NULL;
-}
-
-/*
- * Allocate a new EAP_PACKET
- */
-EAP_DS *eap_ds_alloc(void)
+EAP_DS *eap_ds_alloc(eap_handler_t *handler)
 {
 	EAP_DS	*eap_ds;
 
-	eap_ds = rad_malloc(sizeof(EAP_DS));
-	memset(eap_ds, 0, sizeof(EAP_DS));
-	if ((eap_ds->response = eap_packet_alloc()) == NULL) {
+	eap_ds = talloc_zero(handler, EAP_DS);
+	eap_ds->response = talloc_zero(eap_ds, eap_packet_t);
+	if (!eap_ds->response) {
 		eap_ds_free(&eap_ds);
 		return NULL;
 	}
-	if ((eap_ds->request = eap_packet_alloc()) == NULL) {
+	eap_ds->request = talloc_zero(eap_ds, eap_packet_t);
+	if (!eap_ds->response) {
 		eap_ds_free(&eap_ds);
 		return NULL;
 	}
@@ -110,56 +69,15 @@ void eap_ds_free(EAP_DS **eap_ds_p)
 	eap_ds = *eap_ds_p;
 	if (!eap_ds) return;
 
-	if (eap_ds->response) eap_packet_free(&(eap_ds->response));
-	if (eap_ds->request) eap_packet_free(&(eap_ds->request));
+	if (eap_ds->response) talloc_free(eap_ds->response);
+	if (eap_ds->request) talloc_free(eap_ds->request);
 
-	free(eap_ds);
+	talloc_free(eap_ds);
 	*eap_ds_p = NULL;
 }
 
-/*
- * Allocate a new EAP_HANDLER
- */
-EAP_HANDLER *eap_handler_alloc(rlm_eap_t *inst)
+static int _eap_handler_free(eap_handler_t *handler)
 {
-	EAP_HANDLER	*handler;
-
-	handler = rad_malloc(sizeof(EAP_HANDLER));
-	memset(handler, 0, sizeof(EAP_HANDLER));
-
-	if (inst->handler_tree) {
-		PTHREAD_MUTEX_LOCK(&(inst->handler_mutex));
-		rbtree_insert(inst->handler_tree, handler);
-		PTHREAD_MUTEX_UNLOCK(&(inst->handler_mutex));
-	
-	}
-	return handler;
-}
-
-void eap_opaque_free(EAP_HANDLER *handler)
-{
-	if (!handler)
-		return;
-
-	eap_handler_free(handler->inst_holder, handler);
-}
-
-void eap_handler_free(rlm_eap_t *inst, EAP_HANDLER *handler)
-{
-	if (!handler)
-		return;
-
-	if (inst->handler_tree) {
-		PTHREAD_MUTEX_LOCK(&(inst->handler_mutex));
-		rbtree_deletebydata(inst->handler_tree, handler);
-		PTHREAD_MUTEX_UNLOCK(&(inst->handler_mutex));
-	}
-
-	if (handler->identity) {
-		free(handler->identity);
-		handler->identity = NULL;
-	}
-
 	if (handler->prev_eapds) eap_ds_free(&(handler->prev_eapds));
 	if (handler->eap_ds) eap_ds_free(&(handler->eap_ds));
 
@@ -167,113 +85,80 @@ void eap_handler_free(rlm_eap_t *inst, EAP_HANDLER *handler)
 		handler->free_opaque(handler->opaque);
 		handler->opaque = NULL;
 	}
-	else if ((handler->opaque) && (handler->free_opaque == NULL))
-                radlog(L_ERR, "Possible memory leak ...");
 
 	handler->opaque = NULL;
 	handler->free_opaque = NULL;
 
-	if (handler->certs) pairfree(&handler->certs);
+	/*
+	 *	Give helpful debug messages if:
+	 *
+	 *	we're debugging TLS sessions, which don't finish,
+	 *	and which aren't deleted early due to a likely RADIUS
+	 *	retransmit which nukes our ID, and therefore our stare.
+	 */
+	if (fr_debug_lvl && handler->tls && !handler->finished &&
+	    (time(NULL) > (handler->timestamp + 3))) {
+		WARN("!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!");
+		WARN("!! EAP session with state 0x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x did not finish!                  !!",
+		     handler->state[0], handler->state[1],
+		     handler->state[2], handler->state[3],
+		     handler->state[4], handler->state[5],
+		     handler->state[6], handler->state[7],
+		     handler->state[8], handler->state[9],
+		     handler->state[10], handler->state[11],
+		     handler->state[12], handler->state[13],
+		     handler->state[14], handler->state[15]);
 
-	free(handler);
+		WARN("!! Please read http://wiki.freeradius.org/guide/Certificate_Compatibility     !!");
+		WARN("!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!");
+	}
+	
+	return 0;
 }
 
-
-typedef struct check_handler_t {
-	rlm_eap_t	*inst;
-	EAP_HANDLER	*handler;
-	int		trips;
-} check_handler_t;
-
-static void check_handler(void *data)
+/*
+ * Allocate a new eap_handler_t
+ */
+eap_handler_t *eap_handler_alloc(rlm_eap_t *inst, REQUEST *request)
 {
-	int do_warning = FALSE;
-	uint8_t state[8];
-	check_handler_t *check = data;
+	eap_handler_t	*handler, *old;
+	char buffer[256];
 
-	if (!check) return;
-
-	if (!check->inst || !check->handler) {
-		free(check);
-		return;
+	handler = talloc_zero(NULL, eap_handler_t);
+	if (!handler) {
+		ERROR("Failed allocating handler");
+		return NULL;
 	}
+	handler->inst_holder = inst;
 
-	if (!check->inst->handler_tree) goto done;
+	/* Doesn't need to be inside the critical region */
+	talloc_set_destructor(handler, _eap_handler_free);
 
-	PTHREAD_MUTEX_LOCK(&(check->inst->handler_mutex));
-	if (!rbtree_finddata(check->inst->handler_tree, check->handler)) {
-		goto done;
-	}
+	if (!inst->dedup_tree) return handler;
+
+	if (radius_xlat(buffer, sizeof(buffer), request, inst->dedup_key, NULL, NULL) < 0) return handler;
+
+	handler->dedup = talloc_strdup(handler, buffer);
 
 	/*
-	 *	The session has continued *after* this packet.
-	 *	Don't do a warning.
+	 *	Delete any old handler
 	 */
-	if (check->handler->trips > check->trips) {
-		goto done;
-	}
+	PTHREAD_MUTEX_LOCK(&(inst->session_mutex));
+	old = rbtree_finddata(inst->dedup_tree, handler);
+	if (old) talloc_free(old);
+	PTHREAD_MUTEX_LOCK(&(inst->session_mutex));
 
-	/*
-	 *	No TLS means no warnings.
-	 */
-	if (!check->handler->tls) goto done;
-
-	/*
-	 *	If we're being deleted early, it's likely because we
-	 *	received a transmit from the client that re-uses the
-	 *	same RADIUS Id, which forces the current packet to be
-	 *	deleted.  In that case, ignore the error.
-	 */
-	if (time(NULL) < (check->handler->timestamp + 3)) goto done;
-
-	if (!check->handler->finished) {
-		do_warning = TRUE;
-		memcpy(state, check->handler->state, sizeof(state));
-	}
-
-done:
-	PTHREAD_MUTEX_UNLOCK(&(check->inst->handler_mutex));
-	free(check);
-
-	if (do_warning) {
-		DEBUG("WARNING: !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!");
-		DEBUG("WARNING: !! EAP session for state 0x%02x%02x%02x%02x%02x%02x%02x%02x did not finish!",
-		      state[0], state[1],
-		      state[2], state[3],
-		      state[4], state[5],
-		      state[6], state[7]);
-
-		DEBUG("WARNING: !! Please read http://wiki.freeradius.org/guide/Certificate_Compatibility");
-		DEBUG("WARNING: !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!");
-	}
-}
-
-void eaptype_free(EAP_TYPES *i)
-{
-	cf_section_parse_free(i->cs, i->type_data);
-
-	if (i->type->detach) (i->type->detach)(i->type_data);
-	i->type_data = NULL;
-#ifndef NDEBUG
-	/*
-	 *	Don't dlclose() modules if we're doing memory
-	 *	debugging.  This removes the symbols needed by
-	 *	valgrind.
-	 */
-	if (!mainconfig.debug_memory)
-#endif
-	  if (i->handle) lt_dlclose(i->handle);
-	free(i);
+	return handler;
 }
 
 
 void eaplist_free(rlm_eap_t *inst)
 {
-	EAP_HANDLER *node, *next;
+	eap_handler_t *node, *next;
 
-       	for (node = inst->session_head; node != NULL; node = next) {
+	for (node = inst->session_head; node != NULL; node = next) {
 		next = node->next;
-		eap_handler_free(inst, node);
+		talloc_free(node);
 	}
 
 	inst->session_head = inst->session_tail = NULL;
@@ -296,20 +181,30 @@ static uint32_t eap_rand(fr_randctx *ctx)
 }
 
 
-static EAP_HANDLER *eaplist_delete(rlm_eap_t *inst, EAP_HANDLER *handler)
+static eap_handler_t *eaplist_delete(rlm_eap_t *inst, REQUEST *request,
+				   eap_handler_t *handler)
 {
 	rbnode_t *node;
 
+	if (inst->dedup_tree) (void) rbtree_deletebydata(inst->dedup_tree, handler);
+
 	node = rbtree_find(inst->session_tree, handler);
 	if (!node) return NULL;
 
 	handler = rbtree_node2data(inst->session_tree, node);
 
+	RDEBUG("Finished EAP session with state "
+	       "0x%02x%02x%02x%02x%02x%02x%02x%02x",
+	       handler->state[0], handler->state[1],
+	       handler->state[2], handler->state[3],
+	       handler->state[4], handler->state[5],
+	       handler->state[6], handler->state[7]);
+
 	/*
 	 *	Delete old handler from the tree.
 	 */
 	rbtree_delete(inst->session_tree, node);
-	
+
 	/*
 	 *	And unsplice it from the linked list.
 	 */
@@ -329,10 +224,10 @@ static EAP_HANDLER *eaplist_delete(rlm_eap_t *inst, EAP_HANDLER *handler)
 }
 
 
-static void eaplist_expire(rlm_eap_t *inst, time_t timestamp)
+static void eaplist_expire(rlm_eap_t *inst, REQUEST *request, time_t timestamp)
 {
 	int i;
-	EAP_HANDLER *handler;
+	eap_handler_t *handler;
 
 	/*
 	 *	Check the first few handlers in the list, and delete
@@ -349,8 +244,16 @@ static void eaplist_expire(rlm_eap_t *inst, time_t timestamp)
 		 *	Expire entries from the start of the list.
 		 *	They should be the oldest ones.
 		 */
-		if ((timestamp - handler->timestamp) > inst->timer_limit) {
+		if ((timestamp - handler->timestamp) > (int)inst->timer_limit) {
 			rbnode_t *node;
+
+			RDEBUG("Expiring EAP session with state "
+			       "0x%02x%02x%02x%02x%02x%02x%02x%02x",
+			       handler->state[0], handler->state[1],
+			       handler->state[2], handler->state[3],
+			       handler->state[4], handler->state[5],
+			       handler->state[6], handler->state[7]);
+
 			node = rbtree_find(inst->session_tree, handler);
 			rad_assert(node != NULL);
 			rbtree_delete(inst->session_tree, node);
@@ -365,7 +268,27 @@ static void eaplist_expire(rlm_eap_t *inst, time_t timestamp)
 				inst->session_head = NULL;
 				inst->session_tail = NULL;
 			}
-			eap_handler_free(inst, handler);
+
+#ifdef WITH_TLS
+			/*
+			 *	Remove expired TLS sessions.
+			 */
+			switch (handler->type) {
+			case PW_EAP_TLS:
+			case PW_EAP_TTLS:
+			case PW_EAP_PEAP:
+			case PW_EAP_FAST:
+				tls_fail(handler->opaque); /* MUST be a tls_session! */
+				break;
+
+			default:
+				break;
+			}
+#endif
+
+			talloc_free(handler);
+		} else {
+			break;
 		}
 	}
 }
@@ -376,20 +299,17 @@ static void eaplist_expire(rlm_eap_t *inst, time_t timestamp)
  *	Since we're adding it to the list, we guess that this means
  *	the packet needs a State attribute.  So add one.
  */
-int eaplist_add(rlm_eap_t *inst, EAP_HANDLER *handler)
+int eaplist_add(rlm_eap_t *inst, eap_handler_t *handler)
 {
 	int		status = 0;
 	VALUE_PAIR	*state;
 	REQUEST		*request = handler->request;
 
-	rad_assert(handler != NULL);
-	rad_assert(request != NULL);
-
 	/*
 	 *	Generate State, since we've been asked to add it to
 	 *	the list.
 	 */
-	state = pairmake("State", "0x00", T_OP_EQ);
+	state = pair_make_reply("State", NULL, T_OP_EQ);
 	if (!state) return 0;
 
 	/*
@@ -413,7 +333,7 @@ int eaplist_add(rlm_eap_t *inst, EAP_HANDLER *handler)
 	 */
 	if (rbtree_num_elements(inst->session_tree) >= inst->max_sessions) {
 		status = -1;
-		eaplist_expire(inst, handler->timestamp);
+		eaplist_expire(inst, request, handler->timestamp);
 		goto done;
 	}
 
@@ -432,43 +352,28 @@ int eaplist_add(rlm_eap_t *inst, EAP_HANDLER *handler)
 
 			memcpy(handler->state + i * 4, &lvalue,
 			       sizeof(lvalue));
-		}		
+		}
 	}
 
-	memcpy(state->vp_octets, handler->state, sizeof(handler->state));
-	state->length = EAP_STATE_LEN;
-
 	/*
 	 *	Add some more data to distinguish the sessions.
 	 */
-	state->vp_octets[4] = handler->trips ^ handler->state[0];
-	state->vp_octets[5] = handler->eap_id ^ handler->state[1];
-	state->vp_octets[6] = handler->eap_type ^ handler->state[2];
+	handler->state[4] = handler->trips ^ handler->state[0];
+	handler->state[5] = handler->eap_id ^ handler->state[1];
+	handler->state[6] = handler->type ^ handler->state[2];
+	handler->state[8] = handler->state[2] ^ (((uint32_t) HEXIFY(RADIUSD_VERSION)) & 0xff);
+	handler->state[10] = handler->state[2] ^ ((((uint32_t) HEXIFY(RADIUSD_VERSION)) >> 8) & 0xff);
+	handler->state[12] = handler->state[2] ^ ((((uint32_t) HEXIFY(RADIUSD_VERSION)) >> 16) & 0xff);
 
-	/*
-	 *	and copy the state back again.
-	 */
-	memcpy(handler->state, state->vp_octets, sizeof(handler->state));
+	fr_pair_value_memcpy(state, handler->state, sizeof(handler->state));
 
 	/*
 	 *	Big-time failure.
 	 */
 	status = rbtree_insert(inst->session_tree, handler);
 
-	/*
-	 *	Catch Access-Challenge without response.
-	 */
-	if (inst->handler_tree) {
-		check_handler_t *check = rad_malloc(sizeof(*check));
-
-		check->inst = inst;
-		check->handler = handler;
-		check->trips = handler->trips;
-		request_data_add(request, inst, 0, check, check_handler);
-	}
-
 	if (status) {
-		EAP_HANDLER *prev;
+		eap_handler_t *prev;
 
 		prev = inst->session_tail;
 		if (prev) {
@@ -496,22 +401,25 @@ int eaplist_add(rlm_eap_t *inst, EAP_HANDLER *handler)
 	PTHREAD_MUTEX_UNLOCK(&(inst->session_mutex));
 
 	if (status <= 0) {
-		pairfree(&state);
+		fr_pair_delete_by_num(&request->reply->vps, PW_STATE, 0, TAG_ANY);
 
 		if (status < 0) {
 			static time_t last_logged = 0;
 
 			if (last_logged < handler->timestamp) {
 				last_logged = handler->timestamp;
-				radlog(L_ERR, "rlm_eap: Too many open sessions.  Try increasing \"max_sessions\" in the EAP module configuration");
-			}				       
+				ERROR("rlm_eap (%s): Too many open sessions. Try increasing \"max_sessions\" "
+				      "in the EAP module configuration", inst->xlat_name);
+			}
 		} else {
-			radlog(L_ERR, "rlm_eap: Internal error: failed to store handler");
+			ERROR("rlm_eap (%s): Failed to store handler", inst->xlat_name);
 		}
 		return 0;
 	}
 
-	pairadd(&(request->reply->vps), state);
+	RDEBUG("EAP session adding &reply:State = 0x%02x%02x%02x%02x%02x%02x%02x%02x",
+	       state->vp_octets[0], state->vp_octets[1], state->vp_octets[2], state->vp_octets[3],
+	       state->vp_octets[4], state->vp_octets[5], state->vp_octets[6], state->vp_octets[7]);
 
 	return 1;
 }
@@ -526,19 +434,25 @@ int eaplist_add(rlm_eap_t *inst, EAP_HANDLER *handler)
  *	Also since we fill the eap_ds with the present EAP-Response we
  *	got to free the prev_eapds & move the eap_ds to prev_eapds
  */
-EAP_HANDLER *eaplist_find(rlm_eap_t *inst, REQUEST *request,
-			  eap_packet_t *eap_packet)
+eap_handler_t *eaplist_find(rlm_eap_t *inst, REQUEST *request,
+			  eap_packet_raw_t *eap_packet)
 {
 	VALUE_PAIR	*state;
-	EAP_HANDLER	*handler, myHandler;
+	eap_handler_t	*handler, myHandler;
 
 	/*
 	 *	We key the sessions off of the 'state' attribute, so it
 	 *	must exist.
 	 */
-	state = pairfind(request->packet->vps, PW_STATE);
-	if (!state ||
-	    (state->length != EAP_STATE_LEN)) {
+	state = fr_pair_find_by_num(request->packet->vps, PW_STATE, 0, TAG_ANY);
+	if (!state) {
+		REDEBUG("EAP requires the State attribute to work, but no State exists in the Access-Request packet.");
+		REDEBUG("The RADIUS client is broken.  No amount of changing FreeRADIUS will fix the RADIUS client.");
+		return NULL;
+	}
+
+	if (state->vp_length != EAP_STATE_LEN) {
+		REDEBUG("The RADIUS client has mangled the State attribute, OR you are forcing EAP in the wrong situation");
 		return NULL;
 	}
 
@@ -552,27 +466,47 @@ EAP_HANDLER *eaplist_find(rlm_eap_t *inst, REQUEST *request,
 	 */
 	PTHREAD_MUTEX_LOCK(&(inst->session_mutex));
 
-	eaplist_expire(inst, request->timestamp);
+	eaplist_expire(inst, request, request->timestamp);
 
-	handler = eaplist_delete(inst, &myHandler);
+	handler = eaplist_delete(inst, request, &myHandler);
 	PTHREAD_MUTEX_UNLOCK(&(inst->session_mutex));
 
 	/*
 	 *	Might not have been there.
 	 */
 	if (!handler) {
-		radlog(L_ERR, "rlm_eap: No EAP session matching the State variable.");
+		RERROR("rlm_eap (%s): No EAP session matching state "
+		       "0x%02x%02x%02x%02x%02x%02x%02x%02x",
+		       inst->xlat_name,
+		       state->vp_octets[0], state->vp_octets[1],
+		       state->vp_octets[2], state->vp_octets[3],
+		       state->vp_octets[4], state->vp_octets[5],
+		       state->vp_octets[6], state->vp_octets[7]);
 		return NULL;
 	}
 
 	if (handler->trips >= 50) {
-		RDEBUG2("More than 50 authentication packets for this EAP session.  Aborted.");
-		eap_handler_free(inst, handler);
+		RERROR("rlm_eap (%s): Aborting! More than 50 roundtrips "
+		       "made in session with state "
+		       "0x%02x%02x%02x%02x%02x%02x%02x%02x",
+		       inst->xlat_name,
+		       state->vp_octets[0], state->vp_octets[1],
+		       state->vp_octets[2], state->vp_octets[3],
+		       state->vp_octets[4], state->vp_octets[5],
+		       state->vp_octets[6], state->vp_octets[7]);
+
+
+		talloc_free(handler);
 		return NULL;
 	}
 	handler->trips++;
 
-	RDEBUG2("Request found, released from the list");
+	RDEBUG("Previous EAP request found for state "
+	       "0x%02x%02x%02x%02x%02x%02x%02x%02x, released from the list",
+		state->vp_octets[0], state->vp_octets[1],
+		state->vp_octets[2], state->vp_octets[3],
+		state->vp_octets[4], state->vp_octets[5],
+		state->vp_octets[6], state->vp_octets[7]);
 
 	/*
 	 *	Remember what the previous request was.
